import java.util.*;
import java.io.*;

public class CKmeans {
	static final int MAX = 100;
	static double[][] X;
	static int[][] Y;
	static double centers[][];
	static int[] clusterlabels, old_clusterlabels;
	static int[] clustersize;
	static int[][] constraintmtx;
	static double[][] distance;
	static int nolabels, nofeatures, noexamples, kcenters, noclusterings;
	static double omega, maxdistance;
	static Random generator;

	public static void compute_distance_matrix() {
		maxdistance = Double.MIN_VALUE;
		distance = new double[noexamples][noexamples];
		for (int i=0; i<noexamples; i++)
			for (int j=i+1; j<noexamples; j++) {
				distance[i][j] = 0;
				for (int k=0; k<nofeatures; k++)
					distance[i][j] += (X[i][k]-X[j][k])*(X[i][k]-X[j][k]);
				distance[j][i] = distance[i][j];
				if (distance[i][j] > maxdistance)
					maxdistance = distance[i][j];
			}
	}

	public static void initalization() {
		boolean[] mark = new boolean[noexamples];
		for (int i=0; i<noexamples; i++)
			mark[i] = false;
		int r;
		centers = new double[kcenters][nofeatures];
		for (int i=0; i<kcenters; i++) {
			while (true) {
				r = generator.nextInt(noexamples);
				if (mark[r]==false)
					break;
			}
			for (int j=0; j<nofeatures; j++)
				centers[i][j] = X[r][j];
		}
	}

	public static void recompute_centers() {
		centers = new double[kcenters][nofeatures];
		for (int i=0; i<noexamples; i++)
			for (int j=0; j<nofeatures; j++)
				centers[clusterlabels[i]][j] +=  X[i][j];
		for (int i=0; i<kcenters; i++)
			for (int j=0; j<nofeatures; j++)
				centers[i][j] /= clustersize[i];
	}

	public static int find_cluster_labels(int ex) {
		double min_obj = Double.MAX_VALUE;
		int min_cluster=0;
		double temp, temp_mustlink, temp_cannotlink;
		int count_mustlink, count_cannotlink;
		for (int i=0; i<kcenters; i++) {
			temp = 0;
			for (int j=0; j<nofeatures; j++)
				temp += (centers[i][j]-X[ex][j])*(centers[i][j]-X[ex][j]);

			for (int nex=0; nex<noexamples; nex++)
				if (nex!=ex) {
					//must link
					if (constraintmtx[nex][ex] == 1) {
						if (i != clusterlabels[nex])
							temp += distance[ex][nex];
					}
					//cannot link
					else if (constraintmtx[nex][ex]==-1) {
						if (i == clusterlabels[nex])
							temp += (maxdistance-distance[ex][nex]);
					}
				}

			if (temp < min_obj) {
				min_cluster = i;
				min_obj = temp;
			}
		}
		return min_cluster;
	}

	public static boolean assign_clustering_labels() {
		boolean flag = false;
		int temp;
		clustersize = new int[kcenters];

		for (int i=0; i<noexamples; i++) {
			temp = find_cluster_labels(i);
			if (temp != clusterlabels[i])
				flag = true;
			clusterlabels[i] = temp;
			clustersize[temp]++;
		}
		return flag;
	}

	public static void sampling_constraint(int cindex) {
		constraintmtx = new int[noexamples][noexamples];
		for (int i=0; i<noexamples; i++)
			for (int j=i+1; j<noexamples; j++)
				if (generator.nextDouble()<omega) {
					if (Y[i][cindex]==Y[j][cindex]) {
						constraintmtx[i][j] = 1;
						constraintmtx[j][i] = 1;
					}
					else {
						constraintmtx[i][j] = -1;
						constraintmtx[j][i] = -1;
					}
				}
					
	}

	public static void kmean() {
		clusterlabels = new int[noexamples];
		initalization();

		int iteration = 0;
		while (true) {
			if (!assign_clustering_labels())
				break;
			recompute_centers();
			System.err.print(".");
			iteration++;
			if (iteration==MAX)
				break;	
		}
		System.err.println();

		for (int i=0; i<noexamples; i++)
			System.out.print((clusterlabels[i]+1) + " ");
		System.out.println();
	}	

	public static void main(String[] args) {
                if (args.length != 5) {
                        System.err.println("Usage: java CKmeans file kcenters noclusterings cindex omega");
                        System.exit(1);
                }
		String file = args[0];
		kcenters = Integer.parseInt(args[1]);
		noclusterings = Integer.parseInt(args[2]);
		int cindex = Integer.parseInt(args[3]);
		omega = Double.parseDouble(args[4]);
		generator = new Random();

                System.err.print("Reading in " + file + ":");
                try {
                        BufferedReader in = new BufferedReader(new FileReader(file));
			String str;
			String[] seq;
			str = (in.readLine()).trim();
			seq = str.split(" ");
			noexamples = Integer.parseInt(seq[0]);
			nofeatures = Integer.parseInt(seq[1]);
			nolabels = Integer.parseInt(seq[2]);
			X = new double[noexamples][nofeatures];
			Y = new int[noexamples][nolabels];
			for (int i=0; i<noexamples; i++) {
				str = (in.readLine()).trim();
				seq = str.split(" ");

				for (int j=0; j<nolabels; j++)
					Y[i][j] = Integer.parseInt(seq[j]);
				
				for (int j=nolabels; j<seq.length; j++)
					X[i][j-nolabels] = Double.parseDouble(seq[j]);
				System.err.print(".");
                        }
                        in.close();
                } catch (IOException e) {
                        System.err.print("Can not read " + file + " containing sequence data !!!\n");
                }
		System.err.println("Done reading data.");

		compute_distance_matrix();
		for (int i=0; i<noclusterings; i++) {
			sampling_constraint(cindex);
			kmean();
		}
	}

}
